#!/bin/bash

# ZSC - zfs snapshot cron

CONFIG_FILE="$HOME/.zsc"

FORMAT='+%Y-%m-%d'
DATE=$(date $FORMAT)


### LOCK ###
echo "Locking ZSC"
if ! lockfile-create -p -r 2 "zsc"; then
	echo "Another backup is running, aborting"
	exit 49
fi


split_date() {
	echo "$1" | sed -E 's/([0-9]{4})-([0-9]{2})-([0-9]{2})/\1\n\2\n\3/'
}

handle_dataset() {
	DATASET=$1
	D=$2
	W=$3
	M=$4
	Y=$5

	PREFIX=$6
	if test -z "$PREFIX"; then
		PREFIX="zsc-"
	fi

	if test "!no_prefix!" = "$PREFIX"; then
		PREFIX=""
	fi

	echo "--- $DATASET --- $D $W $M $Y $PREFIX"

	SNAPSHOTS=($(zfs list -t snapshot -o name | grep "$DATASET@$PREFIX" | sort -r))
	echo "Snapshots: ${#SNAPSHOTS[@]}"

	EXISTS=0
	
	for s in "${SNAPSHOTS[@]}"; do
		sdate=$(echo "$s" | awk 'match($0, /.+@'"$PREFIX"'(.+)/, a) {print a[1]}')

		if [ "$sdate" = $DATE ]; then
			EXISTS=1
		fi

		scomp=($(split_date "$sdate") $(date -d "$stripped_date" "+%u %W"))

		echo -n "${scomp[3]} ${scomp[4]} | "
		
		echo -n "${sdate}: "

		if [ $D -gt 0 ]; then
			D=$(( D-1 ))
			echo "KEEP; D: $D"
		elif [ $W -gt 0 ] && [ ${scomp[3]#0} -eq 1 ]; then
			W=$(( W-1 ))
			echo "KEEP; W: $W"
		elif [ $M -gt 0 ] && [ ${scomp[3]#0} -eq 1 ] && [ $(( ${scomp[4]#0} % 4 )) -eq 0 ]; then
			M=$(( M-1 ))
			echo "KEEP; M: $M"
		elif [ $Y -gt 0 ] && [ ${scomp[3]#0} -eq 1 ] && [ ${scomp[4]#0} -eq 4 ]; then
			Y=$(( Y-1 ))
			echo "KEEP; Y: $Y"
		else
			zfs destroy "$s"
			echo "DISCARD"
		fi
	done

	if [ $EXISTS -eq 0 ]; then
		echo -e "Creating snapshot $DATASET@$PREFIX$DATE"
		zfs snapshot $DATASET@$PREFIX$DATE
	fi
}

cat "$CONFIG_FILE" | while IFS=: read dataset recursive d w m y prefix; do
	echo "| ##### Dataset: $dataset Recursive: $recursive Prefix: $prefix d: $d w: $w m: $m y: $y #####"

	if [ $recursive = "true" ]; then
		SUBDATASETS=($(zfs list -o name | grep "$dataset/"))
		echo -e "| Subdatasets: ${SUBDATASETS[@]}"
		echo -e '|'
		for p in "${SUBDATASETS[@]}"; do
			handle_dataset $p $d $w $m $y $prefix | awk '{print "| " $0}'
			echo -e '|'
		done
	fi

	handle_dataset $dataset $d $w $m $y $prefix | awk '{print "| " $0}'

	echo -e "----------------------------------------------------------------------"
done


### UNLOCK ###
echo "Unlocking ZSC"
lockfile-remove "zsc"
